# color.py
# library for doing color manipulations in both rgb and hsb colorspaces
#
#

from math import *
from colorsys import *
from random import random, uniform, randint
from types import GeneratorType

#import rgb

class Color(object):
   def __init__(self, r, g=None, b=None):
      if isinstance(r, (list, tuple, GeneratorType)):
         r, g, b = r
      assert isinstance(r, (int, float)), 'Color needs number, got %s for r' % r
      assert isinstance(g, (int, float)), 'Color needs number, got %s for g' % g
      assert isinstance(b, (int, float)), 'Color needs number, got %s for b' % b
      self.r = r
      self.g = g
      self.b = b
   
   def __iter__(self):
      yield self.r
      yield self.g
      yield self.b
   
   def __nonzero__(self):
      return bool(self.r or self.g or self.b)
   
   def remap(self):
      mx = float(max(self))
      if mx > 1:
         return Color(r/mx for r in self)
      else:
         return self

#   def __str__(self):
#      return rgb.getName(self)

   def __invert__(self):
      return Color(1-x for x in self)

   def __add__(self, other):
      return Color(x+y for x,y in zip(self, other)).remap()

   def __sub__(self, other):
      return Color(x-y for x,y in zip(self, other))

   def getHue(self):
      return rgb_to_hsv(self.r, self.g, self.b)[0]
   def setHue(self, hue):
      self.HSVtoRGB(hue, self.saturation, self.value)
   hue = property(getHue, setHue)

   def getSaturation(self):
      return rgb_to_hsv(self.r, self.g, self.b)[1]
   def setSaturation(self, saturation):
      self.HSVtoRGB(self.value, saturation, self.value)
   saturation = property(getSaturation, setSaturation)

   def getValue(self):
      return rgb_to_hsv(self.r, self.g, self.b)[2]
   def setValue(self, value):
      self.HSVtoRGB(self.hue, self.saturation, value)
   value = property(getValue, setValue)

   def HSVtoRGB(self, hue, saturation, value):
      self.r, self.g, self.b = hsv_to_rgb(hue, saturation, value)

   def jitter(self, a=.2):
      return Color(x + uniform(-a, a) for x in self).remap()

   def getCopy(self):
      return Color(self.r, self.g, self.b)
   copy = property(getCopy)

def cRand():
   return Color(random() for i in range(3))

def cEnemyRand():
   red = random()
   if red > .5:
      green = random()/2
   else:
      green = random()/2 + .5
   return Color(red, green, 0)

def cDiff(c1, c2):
   return sum(abs(x-y) for x,y in zip(c1, c2))

#def getColor(name):
#   return Color(rgb.getColor(name))



# vim: set et sts=3 sw=3:
