#!/usr/bin/python

try:
    import psyco
    psyco.full()
except ImportError:
   pass

from util import *
import powerups
import effects
import enemy
import sound
import player
import options
import hiscore
   
YELLOW = Color(1,1,0)
BLUE = Color(0,0,1)
GREEN = Color(0,1,0)
RED = Color(1,0,0)
pygame.init()


class Game: 
   paused = False
   maxfps = 50.0 # this is not very exact
   msecperf = 1000/maxfps
   splashLocation = Vector(-25, 15, 25)
   
   def __init__(self):
      player.players = player.Players()
      enemy.enemies.init()
      powerups.init()
#      sound.reloadMusic()
      display.resetCamera()
   
   def update(self):
      display.update()
      effects.update()
      powerups.update()
      player.players.update()
      enemy.enemies.update()

   def getCollisions(self):
      player.players.getCollision(enemy.enemies)
      enemy.enemies.getCollision()
        
   def draw(self):
      display.clearAll()
      display.objectCamera()
      display.screenSize = display.getScreenSize()
      if options.background:
         effects.stars.draw()
      enemy.enemies.draw()
      player.players.draw()
      powerups.draw()
      effects.explosion.draw()
      display.wordCamera()
      self.drawWords()
      if display.blur:
         display.fade()
      pygame.display.flip()

   def drawWords(self):
      x, y, z = display.screenSize
#      effects.drawWord(Vector([-45, -33, 10]), str(int(player.players.score)), YELLOW)
#      effects.drawWord(Vector([35, -33, 10]), str(int(player.players.health)), GREEN)
      if options.fps:
         effects.drawWord(Vector([-45, 30, 10]), str(int(timer.clock.get_fps())).zfill(2), BLUE)
      if self.paused:
         effects.drawWord(self.splashLocation, "PAUSED", BLUE)
    
   def handleKeys(self):
      keystate = pygame.key.get_pressed()
      for p in player.players:
          p.vel = Vector(0,0,0)
          if keystate[p.controls["right"]]:
             p.vel.x = p.speed
          if keystate[p.controls["left"]]:
             p.vel.x = -p.speed
          if keystate[p.controls["up"]]:
             p.vel.y = p.speed
          if keystate[p.controls["down"]]:
             p.vel.y = -p.speed
          if keystate[p.controls["shoot"]]:
             if p.alive:
                p.fire(enemy.enemies.ships)
      #testing!!
      if keystate[K_b]:
         global BLUR
         BLUR = True
      if keystate[K_g]:
         player.players.godMode()
      if keystate[K_y]:
         x, y, z = display.screenSize
         powerups.newPowerup(Vector(uniform(-x,x), y, 0))
      if keystate[K_EQUALS]:
         timer.msecPerStep -= .3
      if keystate[K_MINUS]:
         timer.msecPerStep += .3
      if keystate[K_9]:
         display.zoomCamera(-.3)
      if keystate[K_0]:
         display.zoomCamera(.3)
      #end testing
      if keystate[K_p]:
         self.paused = True
         pygame.time.wait(100)
         pygame.mixer.music.pause()
         while self.paused:
            pygame.time.wait(20)
            self.handleEvents()
            keystate = pygame.key.get_pressed()
            timer.tick()
#            effects.update()
            self.draw()
            self.paused = not (keystate[K_p] or keystate[K_SPACE] or keystate[K_ESCAPE])
         pygame.mixer.music.unpause()
         pygame.time.wait(100)
      if keystate[K_ESCAPE]:
         player.players.die()
   
   def handleEvents(self):
      for event in pygame.event.get():
         if event.type == QUIT:
            sys.exit()
         elif event.type == VIDEORESIZE:
            display.reshape(event.w, event.h)
         elif event.type == sound.ENDMUSIC:
            sound.reloadMusic()
      while not pygame.display.get_active():
         pygame.time.wait(100)
         pygame.event.pump()
         timer.clock.tick()
         
   def doTimingStuff(self):
      timer.tick()
      lastStep = timer.clock.get_time()
      if lastStep < self.msecperf and options.limitFramerate:
         pygame.time.wait(int(round(self.msecperf - lastStep)))
     
   def run(self):
      while player.players.areAlive():
         self.handleKeys()
         self.handleEvents()
         self.update()
         self.getCollisions()
         self.draw()
         self.doTimingStuff()
      pygame.mixer.music.fadeout(1)
      EndSplash(player.players.writeScores()).run()
   
    
class Splash(Game):
   def __init__(self):
      if options.background:
         effects.stars.populate()
            
   def update(self):
      effects.update()
      for p in player.players:
         p.zxorientation += .02 * timer.percTicked
         p.vel = Vector(1, 0, 0)
   
   def draw(self):
      display.clearAll()
      display.objectCamera()
      if options.background:
         effects.stars.draw()
      player.players.draw()
      effects.explosion.draw()
      x, y, z = display.screenSize
      display.wordCamera()
      effects.drawWord(self.splashLocation, "Triplex Invaders", GREEN)
      if display.blur:
         display.fade()
      pygame.display.flip()
   
   def run(self):
       while not pygame.key.get_pressed()[K_SPACE]:
           self.handleEvents()
           self.update()
           self.draw()
           self.doTimingStuff()

class EndSplash(Game):
   def __init__(self, hiScoreIndexes):
      self.hiscores = hiscore.readlines()
      self.hiScoreIndexes = hiScoreIndexes
      self.oscillator = timer.Oscillator(3000, .1)

   def drawWords(self):
      x, y, z = display.screenSize
      effects.drawWord(self.splashLocation, "Triplex Invaders", GREEN)
#      effects.drawWord(self.splashLocation + Vector(0,-7,0),
#                        "Score: %s" % self.score, BLUE, .8)
      for i, userscore in enumerate(self.hiscores):
         user, score = userscore
         if i in self.hiScoreIndexes:
            size = 0.5 + self.oscillator.value()
         else:
            size = 0.5
         effects.drawWord(self.splashLocation + Vector(3,-14-3*i ,0),
                 '%s: %d' % (user, score), RED, size)
   
   def run(self):
      while pygame.key.get_pressed()[K_SPACE]:
         self.go()
      while not pygame.key.get_pressed()[K_SPACE]:
         self.go()
      enemy.enemies = enemy.Swarm()
      player.players = player.Players()
      
   def go(self):
      self.handleKeys()
      self.handleEvents()
      effects.update()
      powerups.update()
      player.players.update()
      enemy.enemies.update()
      enemy.enemies.getCollision()
      player.players.getCollision(enemy.enemies)
      self.draw()
      self.doTimingStuff()


if __name__ == '__main__':
   display.invadersLightInit()
   Splash().run()
   while True:
      Game().run()
   


# vim: set et sts=3 sw=3:
