from random import random

from util import *
import sound
import weapon
import effects
import options

def collideShips(shipa, shipb):
   a = getAngle(shipb.pos, shipa.pos)
   d = dist(shipb.pos, shipa.pos)
   effects.explosion.explodeParticles(sqrt(shipa.health + shipb.health)*50,
                             shipb.pos + Vector([d*cos(a), d*sin(a), 0]),
                             vAvg([shipb.vel, shipa.vel]),
                             1, shipa.color+shipb.color)
   bhealth = shipb.health
   ahealth = shipa.health
   shipb.damage(ahealth)
   shipa.damage(bhealth)
   

class Ship(Object):
   rightSpeed = .03
   maxRotation = pi/2
   orientation = 1.5*pi
   zxorientation = 0
   vel = Vector(0,0,0)
   deathNoiseList = [sound.getSound('boom'), sound.getSound('explosion', 0.2)]
   explodeables = []
   lastHitBy = None
   
   def draw(self):
      if self.alive:
         Object.draw(self)
      for w in self.weapons:
         w.draw()
   
   def update(self):
      self.move()
      self.zxorientation = min(self.maxRotation, self.zxorientation)
      self.zxorientation = max(-self.maxRotation, self.zxorientation)
      self.zxorientation -= self.rightSpeed*sin(self.zxorientation)*timer.percTicked
      for w in self.weapons:
         w.update()
   
   def damage(self, a):
      self.health = max(self.health-a, 0)
      if not self.health and self.alive:
         self.die()
   
   def die(self):
      Object.die(self)
      for e in self.explodeables:
         effects.explosion.explodeModel(e)
      effects.explosion.explodeParticles(self.explodeSize, self.pos,
              self.vel/2.0, self.radius, ~self.color)
      effects.explosion.explodeModel(self)
      if sound.isSound:
         choice(self.deathNoiseList).play()
   
   def turn(self, a):
      self.orientation += a
      self.zxorientation -= 3*a*cos(self.zxorientation)


class Player(Ship):
   speed = 1.75
   radius = 5
   score = 0
   health= 100.0
   maxHealth = 100.0
   explodeSize = 1200
   godMode = False
   modelID = 0
   model = getModel('warthog0')
   deathNoiseList = [sound.getSound('bigboom')]
   preEffects = [tilt, effects.playerThruster]
   orientation = pi/2
   
   def __init__(self):
      self.weapons = [weapon.newWeapon()(self)]
#      self.weapons = [weapon.BombGun(self)]
      self.postEffects = [WireModel(self.model)]
      self.lostProjectiles = weapon.lostProjectiles()
      self.pos = Vector(0, -20, 0)
      self.vel = Vector(0, 0, 0)
      self.color = Color(0, 1, 0)
      self.color.value = 1
   
   def draw(self):
      Ship.draw(self)
      self.lostProjectiles.draw()
   
   def update(self):
      Ship.update(self)     
      self.zxorientation += .05*self.vel.x * timer.percTicked
      self.lostProjectiles.update()
      healthPerc = self.health / self.maxHealth
      if healthPerc > 1:
         self.health -= 0.3*timer.percTicked
         self.color.r = 0
         self.color.g = 2 - healthPerc
         self.color.b = healthPerc - 1
      elif healthPerc <= 1:
         self.color.r = 1.0 - healthPerc
         self.color.g = healthPerc
         self.color.b = 0
      x, y, z = display.screenSize
      self.pos.x = min(self.pos.x, x)
      self.pos.x = max(self.pos.x, -x)
      self.pos.y = min(self.pos.y, y)
      self.pos.y = max(self.pos.y, -y)
      #self.color.hue += .001*timer.percTicked
   
   def damage(self, a):
      if not self.godMode:
         Ship.damage(self, self.damageFactor*a)
   
   def fire(self, enemies):
      for w in self.weapons:
         w.fire(self.pos+Vectorp(self.radius, self.orientation),
                 self.orientation, enemies)
   
   def getCollision(self, ships):
      if self.alive:
         for ship in ships:
            if isCollision(self, ship):
               collideShips(self, ship)
      for w in self.weapons:
         w.getCollision(ships)


# vim: set et sts=3 sw=3:
