from math import *
from random import random, uniform, gauss, randint, choice
from types import GeneratorType

class Vector(object):
   def __init__(self, x, y=None, z=None):
      if isinstance(x, (list, tuple, GeneratorType)):
         x, y, z = x
      assert isinstance(x, (int, float)), 'Vector needs number, got %s for x' % x
      assert isinstance(y, (int, float)), 'Vector needs number, got %s for y' % y
      assert isinstance(z, (int, float)), 'Vector needs number, got %s for z' % z
      self.x = x
      self.y = y
      self.z = z
      
   def __str__(self):
#      return '(' + ' + '.join([str(round(x, 2))+y for x,y in zip(self, ['i', 'j', 'k'])]) + ')'
      return '<%s, %s, %s>' % (str(round(self.x, 2)), str(round(self.y, 2)), str(round(self.z, 2)))
      
   def __iter__(self):
      yield self.x
      yield self.y
      yield self.z
   
   def __nonzero__(self):
      return bool(self.x or self.y or self.z)
   
   def __neg__(self):
      return Vector([-x for x in self])
   
   def __add__(self, other):
      return Vector(self.x+other.x, self.y+other.y, self.z+other.z)
   def __iadd__(self, other):
      self.x += other.x
      self.y += other.y
      self.z += other.z
      return self
   
   def __sub__(self, other):
      return Vector(self.x-other.x, self.y-other.y, self.z-other.z)
   def __isub__(self, other):
      self.x-= other.x
      self.y-= other.y
      self.z-= other.z
      return self
   
   def __mul__(self, other):
      if isinstance(other, Vector):
         return Vector(self.x*other.x, self.y*other.y, self.z*other.z)
      elif isinstance(other, (int, float)):
         return Vector(self.x*other, self.y*other, self.z*other)
   def __imul__(self, other):
      if isinstance(other, Vector):
         self.x*= other.x
         self.y*= other.y
         self.z*= other.z
      elif isinstance(other, (int, float)):
         self.x*= other
         self.y*= other
         self.z*= other
      return self
   
   def __div__(self, other):
      if isinstance(other, Vector):
         return Vector(self.x/other.x, self.y/other.y, self.z/other.z)
      elif isinstance(other, (int, float)):
         return Vector(self.x/other, self.y/other, self.z/other)
   def __idiv__(self, other):
      if isinstance(other, Vector):
         self.x/= other.x
         self.y/= other.y
         self.z/= other.z
      elif isinstance(other, (int, float)):
         self.x/= other
         self.y/= other
         self.z/= other
      return self
   
   def getMag(self):
      return sqrt(self.x**2 + self.y**2 + self.z**2)
   def setMag(self, value):
      m = self.getMag()
      assert m != 0
      s = value/m
      self.x *= s
      self.y *= s
      self.z *= s
   mag = property(getMag, setMag)
   
   def getDir(self):
      return atan2(self.y, self.x)
   def setDir(self, value):
      mag = hypot(self.x, self.y)
      self.x = mag*cos(value)
      self.y = mag*sin(value)
   dir = property(getDir, setDir) 
   
   def getZdir(self):
      return atan2(self.z, hypot(self.x, self.y))
   def setZdir(self, value):
      mag = self.getMag() 
      self.z = mag*sin(value)
      if value != 0:
         hyp = self.z / tan(value)
      else: 
         hyp = mag
      dir = self.getDir()
      self.y = sin(dir) * hyp
      self.x = cos(dir) * hyp
   zdir = property(getZdir, setZdir)
   
   def magAlong(self, angle):
      return self.mag * cos(abs(self.dir - angle))
   
   def jitter(self, a):
      return Vector(self.x+uniform(-a, a), self.y+uniform(-a, a), self.z+uniform(-a, a))
   
   def randDir(self):
      return self.jitter(1)
   
   def getCopy(self):
      return Vector(self.x, self.y, self.z)
   copy = property(getCopy)
   
def Vectorp(r, theta):
   return Vector(r*cos(theta), r*sin(theta), 0.0)

def vAvg(vecs):
   return Vector(map(sum, zip(*vecs)))/len(vecs)

def dist(a, b):
   return hypot(a.x-b.x, a.y-b.y)

def getAngle(a, b):
   return atan2(b.y-a.y, b.x-a.x)


# vim: set et sts=3 sw=3:
